#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
A collection of thumbor filters providing better support for color managed
images.
"""

import os

from thumbor.config import Config

# ICC options
_WELL_KNOWN_PATHS = [
    "/usr/share/color/icc"
]

_DEFAULT_PATHS = [path for path in _WELL_KNOWN_PATHS if os.path.exists(path)]

Config.define('ICC_PATH', _DEFAULT_PATHS,
              'An array of path to directories containing ICC profiles.',
              'Color management')
Config.define('ICC_DEFAULT_PROFILE', 'sRGB',
              'Name of the default ICC profile applied of none is specified.',
              'Color management')



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
A collection of thumbor filters providing better support for color managed
images.
"""

import os

from io import BytesIO

from PIL import Image, ImageCms
from thumbor.filters import BaseFilter, filter_method, PHASE_AFTER_LOAD
from thumbor.utils import logger


# pylint: disable=R0903
class Filter(BaseFilter):
    """
    A collection of thumbor filters providing better support for color managed
    images.
    """

    phase = PHASE_AFTER_LOAD

    def _find_profile(self, profile=None):
        result = ""

        if not profile:
            profile = self.context.config.ICC_DEFAULT_PROFILE

        iccfile = profile + ".icc"

        for path in self.context.config.ICC_PATH:
            filepath = os.path.join(path, iccfile)
            logger.debug("ICC: Checking for profile in {:s}".format(filepath))
            if os.path.exists(filepath):
                result = filepath
                break

        return result

    # pylint: disable=R0911
    # pylint: disable=W0703
    @filter_method(BaseFilter.String)
    async def icc_profile_apply(self, profile=None):
        """
        Optain an RGB(A) image by applying the given profile.
        """
        # Check whether input image has color management.
        if not self.engine.icc_profile:
            logger.info("ICC: Image has no embedded profile. "
                        "Skipping this image.")
            return

        # Sanitize profile parameter.
        if profile is not None:
            profile = os.path.basename(profile).lstrip(".")
            if not profile:
                logger.warning("ICC: Invalid profile name.")
                return

        # Find output profile.
        outprofile = self._find_profile(profile)
        if not outprofile:
            logger.warning("ICC: Failed to load profile: {:s}".format(profile))
            return

        try:
            ext = self.engine.extension
            fmt = Image.EXTENSION[ext.lower()]
        except Exception:
            logger.exception("ICC: Failed to determine image format and "
                             "extension before attempting to apply "
                             "profile: {:s}".format(profile))
            return

        try:
            inmode = self.engine.get_image_mode()
            insize = self.engine.size
            indata = self.engine.get_image_data()
            inimg = Image.frombytes(inmode, insize, indata)
            inprofile = BytesIO(self.engine.icc_profile)

            outmode = "RGBA" if "A" in inmode else "RGB"
        except Exception:
            logger.exception("ICC: Failed to determine image "
                             "properties before attempting to apply "
                             "profile: {:s}".format(profile))
            return

        logger.info("ICC: Attempting to apply profile: {:s}, inmode: {:s}, "
                    "outmode: {:s}".format(profile, inmode, outmode))
        try:
            outimg = ImageCms.profileToProfile(inimg, inprofile, outprofile,
                                               outputMode=outmode)
        except Exception:
            logger.exception("ICC: Failed to apply profile: {:s}, "
                             "inmode: {:s}, "
                             "outmode: {:s}".format(profile, inmode, outmode))
            return

        # Reload the image into the engine.
        outbuf = BytesIO()
        try:
            outimg.save(outbuf, fmt)
            self.engine.load(outbuf.getvalue(), ext)
        except Exception:
            logger.exception("ICC: Failed load the image with an applied "
                             "profile: {:s}, inmode: {:s}, "
                             "outmode: {:s}".format(profile, inmode, outmode))
            return
        finally:
            outbuf.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


setup(
    name="thumbor_icc",
    packages=find_packages(),
    version="1.1.1",
    description="Color management filters for thumbor",
    author="Lorenz Schori",
    author_email="lo@znerol.ch",
    keywords=["thumbor", "icc", "images"],
    license="MIT",
    url="https://github.com/znerol/thumbor-icc",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Multimedia :: Graphics :: Presentation"
    ],
    install_requires=[
        "thumbor>7.0.0.dev0",
        "Pillow"
    ],
    long_description="""\
Thumbor is a smart imaging service. It enables on-demand crop, resizing and
flipping of images. This module provide support for color management profiles.
"""
)
